@@ -1,13 +1,8 @@
 import uuid
 
-import lndb_schema_core as schema_core
+import lndb_schema_core as schema
 import sqlmodel as sqm
 from lamin_logger import logger
-
-try:
-    from lamindb.do._load import load
-except ImportError:
-    print("Could not import lamindb.")
 from lndb_setup import settings
 from lndb_setup._hub import connect_hub
 from supabase import Client
@@ -26,40 +21,49 @@ class hub:
     """Access the hub."""
 
     @classmethod
-    def share_instance(cls):
-        """Publish instance with all dobjects."""
-        hub = get_hub_with_authentication()
-        dobject_df = load.entity("dobject")
-        for id, v in dobject_df.index:
-            cls.share_dobject(id, v, hub)
+    def _load_entity(cls, entity):
+        # we need to do this right now as we can't do a cyclic import
+        # we first need to import all of lamindb (including load)
+        # then lndb_hub is imported
+        # once it's imported load can be imported here
+        from lamindb.do._load import load
+
+        return load.entity(entity)
 
     @classmethod
-    def share_dobject(cls, id: str, v: str, hub: Client = None):
-        """Publish a single dobject."""
+    def push_instance(cls):
+        """Push instance with all dobjects."""
+        hub = get_hub_with_authentication()
+        dobject_df = cls._load_entity("dobject")
+        for id, v in dobject_df.index:
+            cls.push_dobject(id, v, hub)
+
+    @classmethod
+    def push_dobject(cls, id: str, v: str = "1", hub: Client = None):
+        """Push a single dobject."""
         if hub is None:
             hub = get_hub_with_authentication()
         instance = get_or_create_instance(hub)
-        dobject = (
-            load.entity("dobject").loc[[(id, v)]].reset_index().to_dict("records")[0]
-        )
-        dsource_id = dobject["dsource_id"]
         with sqm.Session(settings.instance.db_engine()) as session:
-            dtransform = session.get(schema_core.dtransform, dsource_id)
-        if not jupynb_exists(dtransform.jupynb_id, dtransform.jupynb_v, hub):
-            logger.info(
-                f"Published jupynb ({dtransform.jupynb_id}, {dtransform.jupynb_v})."
-            )
-            insert_jupynb(
-                dtransform.jupynb_id, dtransform.jupynb_v, instance["id"], hub
+            dobject = session.get(schema.dobject, (id, v))
+            dtransform = session.get(schema.dtransform, dobject.dsource_id)
+            jupynb = session.get(
+                schema.jupynb, (dtransform.jupynb_id, dtransform.jupynb_v)
             )
+        if not jupynb_exists(jupynb.id, jupynb.v, hub):
+            logger.info(f"Push jupynb ({jupynb.id}, {jupynb.v}).")
+            insert_jupynb(jupynb, instance["id"], hub)
         else:
             logger.warning(
-                f"jupynb ({dtransform.jupynb_id}, {dtransform.jupynb_v}) already exists"
-                " and is not pushed."
+                f"jupynb ({jupynb.id}, {jupynb.v}) already exists and is not pushed."
             )
         if not dobject_exists(id, v, hub):
-            logger.info(f"Published dobject ({id}, {v}).")
-            insert_dobject(id, v, dtransform.jupynb_id, dtransform.jupynb_v, hub)
+            logger.info(f"Push dobject ({id}, {v}).")
+            insert_dobject(dobject, jupynb, hub)
+        else:
+            logger.warning(
+                f"dobject ({dobject.id}, {dobject.v}) already exists and is not pushed."
+            )
         if not user_dobject_exists(id, v, hub):
             insert_user_dobject(id, v, hub)
 
@@ -68,14 +72,14 @@ class hub:
         """Delete the instance on the hub."""
         hub = get_hub_with_authentication()
         instance = get_or_create_instance(hub)
-        dobject_df = load.entity("dobject")
-        jupynb_df = load.entity("jupynb")
+        dobject_df = cls._load_entity("dobject")
+        jupynb_df = cls._load_entity("jupynb")
         for id, v in dobject_df.index:
-            logger.info(f"Deleted dobject ({id}, {v}).")
+            logger.info(f"Delete dobject ({id}, {v}).")
             delete_user_dobject(id, v, hub)
             delete_dobject(id, v, hub)
         for id, v in jupynb_df.index:
-            logger.info(f"Deleted jupynb ({id}, {v}).")
+            logger.info(f"Delete jupynb ({id}, {v}).")
             delete_jupynb(id, v, hub)
         delete_user_instance(instance["id"], hub)
         delete_instance(hub)
@@ -88,7 +92,7 @@ def get_or_create_instance(hub: Client):
     instance = get_instance(hub)
     if instance is None:
         instance = insert_instance(hub)
-        logger.info("Publishing instance.")
+        logger.info("Pushing instance.")
     if not user_instance_exists(instance["id"], hub):
         insert_user_instance(instance["id"], hub)
     return instance
@@ -185,20 +189,19 @@ def delete_jupynb(id, v, hub: Client):
         hub.table("jupynb").delete().eq("id", id).eq("v", v).execute()
     except Exception:
         logger.warning(
-            "Did not delete jupynb ({id}, {v}) probably because it's still attached to"
+            f"Did not delete jupynb ({id}, {v}) probably because it's still attached to"
             " another dobject."
         )
 
 
-def insert_jupynb(id, v, instance_id, hub: Client):
-    jupynb = load.entity("jupynb").loc[[(id, v)]].reset_index().to_dict("records")[0]
+def insert_jupynb(jupynb: schema.jupynb, instance_id: str, hub: Client):
     data = (
         hub.table("jupynb")
         .insert(
             {
-                "id": jupynb["id"],
-                "v": jupynb["v"],
-                "name": jupynb["name"],
+                "id": jupynb.id,
+                "v": jupynb.v,
+                "name": jupynb.name,
                 "type": "nbproject",
                 "instance_id": instance_id,
                 "owner_id": hub.auth.session().user.id.hex,
@@ -207,8 +210,8 @@ def insert_jupynb(id, v, instance_id, hub: Client):
                 # works because RLS policy. So for the moment the owner_id is
                 # the id of the first user that push a dobject from this
                 # notebook.
-                "time_created": jupynb["time_created"].strftime("%Y-%m-%d %H:%M:%S"),
-                "time_updated": jupynb["time_updated"].strftime("%Y-%m-%d %H:%M:%S"),
+                "time_created": jupynb.time_created.strftime("%Y-%m-%d %H:%M:%S"),
+                "time_updated": jupynb.time_updated.strftime("%Y-%m-%d %H:%M:%S"),
             }
         )
         .execute()
@@ -231,20 +234,19 @@ def delete_dobject(id, v, hub: Client):
     hub.table("dobject").delete().eq("id", id).eq("v", v).execute()
 
 
-def insert_dobject(id, v, jupynb_id, jupynb_v, hub: Client):
-    dobject = load.entity("dobject").loc[[(id, v)]].reset_index().to_dict("records")[0]
+def insert_dobject(dobject: schema.dobject, jupynb: schema.jupynb, hub: Client):
     data = (
         hub.table("dobject")
         .insert(
             {
-                "id": dobject["id"],
-                "v": dobject["v"],
-                "name": dobject["name"],
-                "file_suffix": dobject["file_suffix"],
-                "jupynb_id": jupynb_id,
-                "jupynb_v": jupynb_v,
-                "time_created": dobject["time_created"].strftime("%Y-%m-%d %H:%M:%S"),
-                "time_updated": dobject["time_updated"].strftime("%Y-%m-%d %H:%M:%S"),
+                "id": dobject.id,
+                "v": dobject.v,
+                "name": dobject.name,
+                "file_suffix": dobject.file_suffix,
+                "jupynb_id": jupynb.id,
+                "jupynb_v": jupynb.v,
+                "time_created": dobject.time_created.strftime("%Y-%m-%d %H:%M:%S"),
+                "time_updated": dobject.time_updated.strftime("%Y-%m-%d %H:%M:%S"),
             }
         )
         .execute()
